--[[
	UO_OGRANISER:
	'uo' (unOfficial) Version by Hoshizora [http://hoshizora.tripod.com/uo_ograniser]
	Original Concept and Program by modsyn [http://cs.ulm.edu/~jself]

	Uses LUA Player: 0.20 MOD 2 by Cools [http://www.theriddlingsphinx.com/]
	Last updated   : 2006.07.21
  ]]

----------------------
-- Import Libraries --
----------------------
require('lib/pad')
require('lib/string')

-----------------------
-- Utility Functions --
-----------------------
--Hoshizora 2006.07.11: New ogranise screen with progress bar! :)
function osTouchProgressScreen(shortcutdir,curri,maxi,msState)
	local pbarwidth=300
	local pbarheight=fontdim+1
	local pbarcurrwidth=pbarwidth/maxi*(curri-1)
	local pbarpercent=math.floor((curri-1)/maxi*100)..'%'

	os:print(fontdim*11,fontdim*13,'Ogranising: '..shortcutdir,black)
	os:print(fontdim*11,fontdim*14,'Completed '..curri-1 ..' out of '..maxi,black)
	os:print(fontdim*11,fontdim*19,msState,black)
	if msState=='Done.' then
		os:print(fontdim*11,fontdim*20,'Press   to view report.',black)
	else
		os:print(fontdim*11,fontdim*20,'Do not remove the Memory Stick.',black)
	end

	--Draw the progress bar!
	os:fillRect(fontdim*11,fontdim*16-2,pbarwidth,pbarheight+2,black) -- Outer border
	os:fillRect(fontdim*11+1,fontdim*16-1,pbarwidth-2,pbarheight) -- Leave 1 px for the border
	os:fillRect(fontdim*11+1,fontdim*16-1,pbarcurrwidth-2,pbarheight,green) -- The bar itself!
	os:print((screenwidth-fontdim*string.len(pbarpercent))/2,fontdim*16,pbarpercent,black) -- Percentage completed

	screen:blit(0,0,contextbg,0,0,contextbg:width(),contextbg:height(),false)
	osPrintBatteryLife()
	screen:blit(0,0,os)
	if msState=='Done.' then
		screen:blit(fontdim*17,fontdim*20,padcross)
	end
	screen.waitVblankStart()
	screen.flip()
end

--Hoshizora 2006.07.07: Modified function touch_icon to support file counter
--Hoshizora 2006.07.12: Rewrote the touch routine to support for multiple files within the shortcut dir
--Hoshizora 2006.07.15: Support for directory renaming to repair/restore corrupted icons
function touch_shortcut(proglist,shortcutdir,repair,curri,maxi)
	local files={}
	local shortcutfiles={}
	local oldProgDir=''
	local oldShortcutDir=''
	local newProgDir=''
	local key=''
	local keylen=3
	local success=true
	local errcode=0
	local inf,outf,msg,block,filedata,ok,retry

	local logfile=io.open(logfiledir,'a+') -- Log this operation
	if debug then logfile:write(pad_right('BEGIN: Shortcut '..shortcutdir,maxtitlelen)..'\n') end

	osClear()
	osTouchProgressScreen(shortcutdir,curri,maxi,'Reading...')
	files=System.listDirectory(gamedir..shortcutdir)
	for index,file in ipairs(files) do
		if not (file.directory or file.name=='.' or file.name=='..') then
			-- Read a file to memory
			inf,msg=io.open(gamedir..shortcutdir..'/'..file.name,'rb')
			if inf then
				block=''
				filedata={}
				repeat
					block=inf:read(1024*8)
					table.insert(filedata,block)
				until block==nil
				inf:close()
				table.insert(shortcutfiles,{name=file.name,data=filedata})

				-- Remove the file from MS after the read
				System.removeFile(gamedir..shortcutdir..'/'..file.name)
				if fileExists(gamedir..shortcutdir..'/'..file.name) then -- Assertion check
					if debug then logfile:write(pad_right('  System.removeFile('..file.name..')',maxtitlelen)..'  Error\n') end
					success=false
					errcode=1
				end
			else
				if debug then logfile:write(pad_right('  io.open('..file.name..',rb): '..msg,maxtitlelen)..'  Error\n') end
				success=false
				errcode=errcode+2
			end
		end
	end

	-- Remove the shortcut directory (after all the files have been processed)
	System.removeDirectory(gamedir..shortcutdir)
	if dirExists(gamedir,shortcutdir) then -- Assertion check
		if debug then logfile:write(pad_right('  System.removeDirectory()',maxtitlelen)..'  Error\n') end
		success=false
		errcode=errcode+4
	end

	-- Update the progress screen
	osClear()
	osTouchProgressScreen(shortcutdir,curri,maxi,'Saving...')

	-- Create the (new) shortcut directory, aka 'touching' the directory and modifying the timestamp
	if repair==1 then -- Repair corrupted data icons, rename existing dirname/dirname% to __SCE__dirname/%__SCE__dirname
		ok=true
		oldShortcutDir=shortcutdir
		if (string.startsWith(shortcutdir,'%') or string.endsWith(shortcutdir,'%')) and not string.endsWith(shortcutdir,'~1%') then
			-- Fixes dirname% and %(_)sce(_)dirname errors
			ok=false
			for i=1,table.getn(proglist) do -- Assert shortcut name have a matching program name
				if string.upper(proglist[i])==string_format(shortcutdir,2) then -- We have a match!
					oldProgDir=proglist[i]
					newProgDir='__SCE__'..string_format(oldProgDir,0)
					shortcutdir='%'..newProgDir
					ok=true
					break
				end
			end
		end
		if ok and string.upper(newProgDir)~=string.upper(oldProgDir) then --If newProgDir==oldProgDir, then the program dir is already repaired, so skip this section
			-- Let's do a check on the new dir names to make sure the names are unique and not already in use, if not unique, we'll fix it
			retry=0
			repeat
				shortcutdir='%'..newProgDir..key
				if dirExists(gamedir,newProgDir..key) or dirExists(gamedir,shortcutdir) then
					key=getUniqueKey(keylen) -- Try to obtain a unique key to append to the dir name, key is empty if dir name is already unique
					retry=retry+1
					ok=false
				else
					ok=true
				end
			until ok or retry==3
			if ok then
				-- The program dir simply needs to be renamed to match the shortcut dir name
				System.rename(gamedir..oldProgDir,gamedir..newProgDir..key) -- Requires Lua Player .20 and above to work
				if dirExists(gamedir,oldProgDir) then -- Assertion check
					if debug then logfile:write(pad_right('  repair=1: System.rename(), key='..key,maxtitlelen)..'  Error\n') end
					shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
					success=false
					errcode=errcode+8
				end
			else
				if debug then logfile:write(pad_right('  repair=1: Unique key not found',maxtitlelen)..'  Error\n') end
				shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
				success=false
				errcode=errcode+16
			end
		elseif not ok then
			if debug then logfile:write(pad_right('  repair=1: program=shortcut not found',maxtitlelen)..'  Error\n') end
			shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
			success=false
			errcode=errcode+32
		end

	elseif repair==2 then -- Restore corrupted data icons, rename existing __SCE__dirname/%__SCE__dirname to dirname/dirname%
		ok=true
		oldShortcutDir=shortcutdir
		if string.startsWith(shortcutdir,'%') then
			ok=false
			for i=1,table.getn(proglist) do -- Assert shortcut name have a matching program name
				if string.upper(proglist[i])==string_format(shortcutdir,2) then -- We have a match!
					oldProgDir=proglist[i]
					newProgDir=string_format(oldProgDir,0)
					shortcutdir=newProgDir..'%'
					ok=true
					break
				end
			end
		end
		if ok and string.upper(newProgDir)~=string.upper(oldProgDir) then --If newProgDir==oldProgDir, then the program dir is already restored, so skip this section
			-- Let's do a check on the new dir names first
			retry=0
			repeat
				shortcutdir=newProgDir..key..'%'
				if dirExists(gamedir,newProgDir..key) or dirExists(gamedir,shortcutdir) then
					key=getUniqueKey(keylen) -- Try to obtain a unique key to append to the dir name, key is empty if dir name is already unique
					retry=retry+1
					ok=false
				else
					ok=true
				end
			until ok or retry==3
			if ok then
				-- The program dir simply needs to be renamed
				System.rename(gamedir..oldProgDir,gamedir..newProgDir..key) -- Requires Lua Player .20 and above to work
				if dirExists(gamedir,oldProgDir) then -- Assertion check
					if debug then logfile:write(pad_right('  repair=2: System.rename(), key='..key,maxtitlelen)..'  Error\n') end
					shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
					success=false
					errcode=errcode+64
				end
			else
				if debug then logfile:write(pad_right('  repair=2: Unique key not found',maxtitlelen)..'  Error\n') end
				shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
				success=false
				errcode=errcode+128
			end
		elseif not ok then
			if debug then logfile:write(pad_right('  repair=2: program=shortcut not found',maxtitlelen)..'  Error\n') end
			shortcutdir=oldShortcutDir -- Rename failed, so revert new shortcut dir to old shortcut dir name
			success=false
			errcode=errcode+256
		end
	end
	
	-- Create the shortcut dir using the original/fixed name
	System.createDirectory(gamedir..shortcutdir)
	if not dirExists(gamedir,shortcutdir) then -- Assertion check
		if debug then logfile:write(pad_right('  System.createDirectory()',maxtitlelen)..'  Error\n')	end
		success=false
		errcode=errcode+512
	end

	-- Copy the files in memory back to the shortcut directory
	for index,file in ipairs(shortcutfiles) do
		outf,msg=io.open(gamedir..shortcutdir..'/'..file.name,'wb')
    if outf then
			for idx,dat in pairs(file.data) do
				outf:write(dat)
			end
			outf:flush()
			outf:close()
		else
			if debug then logfile:write(pad_right('  io.open('..file.name..',wb): '..msg,maxtitlelen)..'  Error\n') end
			success=false
			errcode=errcode+1024
		end
	end

	if debug then logfile:write(pad_right('END: Shortcut '..shortcutdir,maxtitlelen)..'  '..print_result(success)..'\n') end
	msg=string_format(shortcutdir,0)
	if not debug then
		if success then
			logfile:write(pad_right(msg,maxtitlelen)..'  OK\n')
		else
			logfile:write(pad_right(msg,maxtitlelen)..'  Error Code: '..errcode..'\n')
		end
	end
	logfile:flush()
	logfile:close()
  System.sleep(1000)

	-- Update the progress screen
	if curri==maxi then
		osClear()
		osTouchProgressScreen('',curri+1,maxi,'Done.')
		waitX()
	end
end

function getUniqueKey(keylen)
	local key='_'
	for i=1,keylen do
		key=key..math.random(0,9)
	end
	return key
end

function print_result(ok)
	if ok then return 'OK' end
	return 'Failed'
end

--Hoshizora 2006.07.13: Append white spaces to the right get a nicely formatted table effect
function pad_right(str,maxlen)
	local white=' '
	local pad=''
	local strlen=string.len(str)
	if strlen>maxlen then -- concatenate string
		return string.sub(str,1,maxlen-2)..'..'
	elseif strlen<maxlen then
		for i=1,maxlen-strlen do
			pad=pad..white
		end
	end
	return str..pad
end

--Hoshizora 2006.07.14: Append white spaces to the left to get a nicely formatted table effect
function pad_left(str,maxlen)
	local white=' '
	local pad=''
	local strlen=string.len(str)
	if strlen>maxlen then -- concatenate string
		return string.sub(str,1,maxlen-2)..'..'
	elseif strlen<maxlen then
		for i=1,maxlen-strlen do
			pad=pad..white
		end
	end
	return pad..str
end

--Hoshizora 2006.07.13: New splash screen! :)
function osSplashScreen()
	os:clear()
	os:print(fontdim,fontdim*28,'Press',black)
	os:print(fontdim,fontdim*31,'\'uo\' (unOfficial) Version by Hoshizora',black)
	os:print(fontdim,fontdim*32,'Original Concept and Program by modsyn',black)
	os:print(fontdim*49,fontdim*32,pad_left(progver,10),black)
	screen:blit(0,0,splashbg,0,0,splashbg:width(),splashbg:height(),false)
	screen:blit(0,0,os)
	screen:blit(fontdim*7,fontdim*28,padstart)
	screen.waitVblankStart()
	screen.flip()
  waitSTART()
end

--Hoshizora 2006.07.14: -- New report screen! :)
function osReportScreen()
	local lines={}
	local idx=1
	local ss=24
	local inf,msg,tlen,ans

	-- Read the log file to memory
	inf,msg=io.open(logfiledir,'r')
	if inf then
		for line in inf:lines() do
			table.insert(lines,line)
		end
		inf:close()
	end

	cprint_report(reporttitle,lines,idx,ss,msgReportScreen)
	tlen=table.getn(lines)
  while true do
	  Pad:Update()
		if Pad.tLr and tlen>ss then -- Use this only in a multi-page report
			-- Pushed RIGHT
			if idx+ss<=tlen then -- Go to next page
				idx=idx+ss
			end
			cprint_report(reporttitle,lines,idx,ss,msgReportScreen)
		elseif Pad.tLl and tlen>ss then -- Use this only in a multi-page report
			-- Pushed LEFT
			if idx-ss>=1 then -- Go to prev page
				idx=idx-ss
			end
			cprint_report(reporttitle,lines,idx,ss,msgReportScreen)
		elseif Pad.Ax<(analogDeadZone*-1) then
			-- Pushed LEFT (Analog Stick)
			ans=confirm(msgYNQuitOgraniser)
			if ans then
				System.Quit() -- Requires Lua Player .17dk_2 / .20 MOD and above to work
			end
			cprint_report(reporttitle,lines,idx,ss,msgReportScreen)
		elseif Pad.St then
			break
		elseif Pad.Sl then
			screen:save(screencapdir)
			cprint_report(reporttitle,lines,idx,ss,msgScreenCaptured)
		end
	end
end

--Hoshizora 2006.07.14: Helper function to print the report list
function cprint_report(title,lines,idx,ss,statusBarMsg)
	local tlen,totpages,currpage
  
	osClear()
  os:print(fontdim,fontdim,title,yellow)
	os:print(fontdim*3,fontdim*3,pad_right('Title',maxtitlelen)..'  Result',blue)
	tlen=table.getn(lines)
  for i=idx,(idx+ss-1) do
    if i>tlen then break end -- eof
		os:print(fontdim*3,fontdim*(i-idx+4),lines[i],white)
  end

	os:print(fontdim*3,fontdim*32,statusBarMsg,white)
  screen:blit(0,0,reportbg,0,0,reportbg:width(),reportbg:height(),false)
	osPrintBatteryLife()

	-- Get total no. of pages
	totpages=math.ceil(tlen/ss)
	currpage=math.ceil(idx/ss)
	if totpages>1 then
		os:print(fontdim*46,fontdim*29,pad_left('Page '..currpage..'/'..totpages,10),black)
	end
	screen:blit(0,0,os)
	osPrintReportScreenPad(statusBarMsg)
	if totpages>1 and currpage<=totpages and currpage~=1 then
		screen:blit(fontdim*57,fontdim*29,padleft)
	end		
	if totpages>1 and currpage>=1 and currpage~=totpages then
		screen:blit(fontdim*58,fontdim*29,padright)
	end
	screen.waitVblankStart()
  screen.flip()
end

function osPrintReportScreenPad(msg)
	if msg==msgReportScreen then
		screen:blit(fontdim*9,fontdim*32,padstart)
		screen:blit(fontdim*43,fontdim*32,analogleft)
	end
end

--Hoshizora 2006.07.11: Recycle the table by removing all the elements in the table
function table.del(list)
	local n=table.getn(list)
	if n==0 then return end
	for i=1,n do
		table.remove(list)
	end
end

function getGameList(prog1_5,list1_5,list1_0)
	local	files=System.listDirectory(gamedir)
	local dir

	-- 1st pass: Separate dir names with % in 1.5 list and those without in 1.0 list
	table.del(prog1_5)
	table.del(list1_5)
	table.del(list1_0)
	for index,file in ipairs(files) do
		if file.directory and string.startsWith(file.name,'%') or string.endsWith(file.name,'%') then
			table.insert(list1_5,1,file.name)
		elseif file.directory and not (file.name=='.' or file.name=='..' or string.match(file.name,'[ _]+[1_]$')) then
			table.insert(list1_0,file.name)
		end
	end
	-- 2nd pass: Sort the dir names in 1.0/1.5 list
	if table.getn(list1_0)==0 then return end -- Nothing to remove!
	for i=1,table.getn(list1_5) do
		for j=1,table.getn(list1_0) do
			if string_format(list1_5[i],2)==string.upper(list1_0[j]) then -- We have a match!
				dir=table.remove(list1_0,j) -- Remove the 1.5 dir in the 1.0 list
				table.insert(prog1_5,dir) -- Insert the 1.5 dir to the 1.5 program list
				break -- We can exit the inner loop
			end
		end
	end
end

-- Hoshizora 2006.07.08: Added status bar functionality. cprint() will do most of the GUI work for the Main Menu
function cprint(title,lines,idx,hide,repair,statusBarMsg)
	local ss=17 -- no. of items to display in the list + 1
  local st=0
	local max=1
	local sh=-2
	local tlen,p,color

	--max isnt set then set it
  if max==1 then max=ss+1 end
	--up max if index is greater
  if idx>max then max=idx end
	--index got larger than max so set start point
  st=max-ss
	--adjust max when index goes low
  if idx<st then
    max=st+ss-1
    st=idx
  end
	--reset max when start is reset
  if st==1 then max=ss+1 end

	osClear()
	os:print(fontdim,fontdim,title,yellow)

  -- Print the list
	tlen=table.getn(lines)
	for i=st,(st+ss) do
    if i>tlen then break end
    p=fontdim*(i-st+3)
    if i==idx then color=blue
    else color=white
    end
		-- Hoshizora 2006.07.08: Pre-format homebrew titles before printing to screen
		if hide then
			os:print(fontdim*3,p,string_format(lines[i],0),color)
		else
			os:print(fontdim*3,p,lines[i],color)
		end
  end

	-- Hoshizora 2006.07.10: Updated screen instructions and display inner box as transparent
  os:print(fontdim*3,fontdim*24+sh,' Manual Sorting: Hold   +  /  On Selection',white)
  os:print(fontdim*3,fontdim*25+sh,'   Auto Sorting:   Ascending          Descending',white)
  os:print(fontdim*3,fontdim*26+sh,' Listing Option:   FW1.0 List         FW1.5 List',white)
	os:print(fontdim*3,fontdim*27+sh,'                   Restore Original FW1.5 List',white)
	os:print(fontdim*3,fontdim*28+sh,'KXploit Folders:          XMB Corrupted Icons:',white) 
	if hide then
  os:print(fontdim*22,fontdim*28+sh,'HIDE',yellow)
	else
	os:print(fontdim*22,fontdim*28+sh,'SHOW',yellow)
	end
	if repair==1 then
	os:print(fontdim*52,fontdim*28+sh,'HIDE',yellow)
	elseif repair==2 then
	os:print(fontdim*52,fontdim*28+sh,'SHOW',yellow)
	else
	os:print(fontdim*52,fontdim*28+sh,'----',yellow)
	end
	os:print(fontdim*3,fontdim*29+sh,'   Quit            Screen Capture           Ogranise',white)
	os:print(fontdim*3,fontdim*32, statusBarMsg, white)
  screen:blit(0,0,mainbg,0,0,mainbg:width(),mainbg:height(),false)
	osPrintBatteryLife()
	osPrintListMode(title)
	screen:blit(0,0,os)
	screen:blit(fontdim*25,fontdim*24+sh,padcross)
	screen:blit(fontdim*29,fontdim*24+sh,padup)
	screen:blit(fontdim*31,fontdim*24+sh,paddown)
	screen:blit(fontdim*20,fontdim*25+sh,padtriangle)
	screen:blit(fontdim*39,fontdim*25+sh,padsquare)
	screen:blit(fontdim*20,fontdim*26+sh,padleft)
	screen:blit(fontdim*39,fontdim*26+sh,padright)
	screen:blit(fontdim*20,fontdim*27+sh,padcircle)
	screen:blit(fontdim*20,fontdim*28+sh,padl)
	screen:blit(fontdim*50,fontdim*28+sh,padr)
	screen:blit(fontdim*18,fontdim*29+sh,padselect)
	screen:blit(fontdim*43,fontdim*29+sh,padstart)
	screen:blit(fontdim*3,fontdim*29+sh,analogleft)
	screen.waitVblankStart()
  screen.flip()
end

-- Hoshizora 2006.07.08: Added cprint2 function to decide which message to print in original/current listing
function cprint2(title,list,sel,move,hide,repair,msgOrig,msgCurr)
	if move then -- Current listing
		cprint(title,list,sel,hide,repair,msgCurr)
	else -- Original listing
		cprint(title,list,sel,hide,repair,msgOrig)
	end
end

function osPrintProgramInfo()
	-- Hoshizora 2006.07.08: Display program information
	local proginfo=progname .. ' ' .. progver
	os:print((screenwidth-fontdim*string.len(proginfo))/2,fontdim,proginfo,yellow)
end

function osPrintListMode(mode)
	local shx=5
	local shy=3
	if mode==list1_0title then
		screen:blit(fontdim*57+shx,fontdim*32-shy,iconlistlocked)
	else
		screen:blit(fontdim*57+shx,fontdim*32-shy,iconlistunlocked)
	end
end

function osPrintBatteryLife()
	-- Hoshizora 2006.07.16: Display battery life indicator
	local bbarwidth=13
	local bbarheight=8
	local bbarpercent=System.powerGetBatteryLifePercent()
	local bbareraserwidth=math.floor(bbarwidth/100*(100-bbarpercent))
	local icoshx=7
	local icoshy=-2
	local barshx=4
	local barshy=2
	
	--Draw the battery life bar!
	if bbarpercent>=50 then -- The battery life indicator
		os:fillRect(fontdim*56+icoshx+barshx,fontdim+icoshy+barshy,bbarwidth,bbarheight,green)
	elseif bbarpercent>=15 and bbarpercent<50 then
		os:fillRect(fontdim*56+icoshx+barshx,fontdim+icoshy+barshy,bbarwidth,bbarheight,yellow)
	else
		os:fillRect(fontdim*56+icoshx+barshx,fontdim+icoshy+barshy,bbarwidth,bbarheight,red)
	end
	os:fillRect(fontdim*56+icoshx+barshx,fontdim+icoshy+barshy,bbareraserwidth,bbarheight) -- The bar eraser!
	screen:blit(fontdim*56+icoshx,fontdim+icoshy,iconbattery)
end

-- Hoshizora 2006.07.11: Changed function name from clear() to osClear()
function osClear()
		-- Hoshizora 2006.07.10: Updated function to accomodate using image as background
		os:clear()
		osPrintProgramInfo()
end

-- Hoshizora 2006.07.10: Confirm screen before executing the touch process
function confirm(msg)
	local msgprompt='  Yes      No'

  while true do
		osClear()
		os:print((screenwidth-fontdim*string.len(msg))/2, fontdim*15, msg, black)
		os:print((screenwidth-fontdim*string.len(msgprompt))/2, fontdim*19, msgprompt, white)
    screen:blit(0,0,contextbg,0,0,contextbg:width(),contextbg:height(),false)
		osPrintBatteryLife()
		screen:blit(0,0,os)
		screen:blit((screenwidth-fontdim*string.len(msgprompt))/2, fontdim*19,padcross)
		screen:blit((screenwidth-fontdim*string.len(msgprompt))/2+fontdim*9, fontdim*19,padcircle)
		screen.waitVblankStart()
    screen.flip()
    Pad:Update()
    if Pad.tRd then
			-- Pushed CROSS
			osClear()
			return true
    elseif Pad.tRr then
			-- Pushed CIRCLE
			osClear()
			return false
    end
  end
end

function waitSTART()
  while true do
    Pad:Update()
    if Pad.tSt then
			break
		elseif Pad.tSl then
			-- Pushed Select
			-- Hoshizora 2006.07.20: Added screen capture functionality to the splash screen
			screen:save(screencapdir)
		end
  end
end

function waitX()
  while true do
    Pad:Update()
    if Pad.Rd then break end
  end
end

function fileExists(file)
  local t,msg=io.open(file,"r")
		if not t then return false end
		t:close()
  return true
end

-- Hoshizora 2006.07.15: Check if directory exists
function dirExists(dir,name)
	local files=System.listDirectory(dir)
	local ok=false
	for index,file in ipairs(files) do
		if file.directory and string.upper(file.name)==string.upper(name) then
			ok=true
			break -- Dir exists, let's return the result
		end
	end
	return ok
end

function swap(t,i,j)
  local tmp=t[i]
  t[i]=t[j]
  t[j]=tmp
end

-- Hoshizora 2006.07.11: Added string_format function to preformat the string for the string.sort function
-- Hoshizora 2006.07.15: type=0-No case conversion, loose sc.cf; type=1-Case conversion, loose sc.cf; type=2-Case conversion, strict sc.cf-w/sce
-- Hoshizora 2006.07.19: type=3-No case conversion, loose prg.cf; type=4-Case conversion, loose prg.cf
-- Hoshizora 2006.07.20: Added 
function string_format(str,type)
	local s=''
	local strf=''
	--local shortcutPattern={' 1',' +[1_]$','~1%','%','^%%?_+[sS][cC][eE]_*'}
	local shortcutPattern={'[ _]+[1_]$','~1%','%','^%%?_+[sS][cC][eE]_*'}

	if type==1 or type==2 or type==4 then -- Convert string to all caps
		s=string.upper(str)
	else -- if type==0 then, no case conversion on the string
		s=str
	end
	slen=string.len(s)
	if string.endsWith(s,shortcutPattern[2]) then
		return string.sub(s,1,slen-4)
	elseif string.endsWith(s,shortcutPattern[3]) then
		return string.sub(s,1,slen-1)
	elseif type==2 and string.startsWith(s,shortcutPattern[3]) then
		return string.sub(s,2,slen)
	elseif (type==3 or type==4) and string.match(s,shortcutPattern[1]) then
		strf=string.gsub(s,shortcutPattern[1],'')
		return string_format2(s,strf)
	else
		strf=string.gsub(s,shortcutPattern[4],'')
		return string_format2(s,strf)
	end
end

function string_format2(s,strf)
	if string.len(strf)==0 then -- Error in the formatted string, just return the original string
		return s
	end
	return strf
end

function tableSortAsc(list)
	table.sort(list, function(a,b)
		return (string_format(a,1)<string_format(b,1))
	end)
end

function tableSortDesc(list)
	table.sort(list, function(a,b)
		return (string_format(a,1)>string_format(b,1))
	end)
end

function restoreOriginalList(list,olist)
	for i=1,table.getn(list) do
		table.remove(list)
	end
	for i=1,table.getn(olist) do
		table.insert(list,olist[i])
	end
end

function choose(title,list1_5,list1_0)
	local sel=1
	local tmp=''
	local olist1_5={}
	local show=list1_5
	local move=true
	local hide=true
	local repair=0
	local msg,ans

	-- Hoshizora 2006.07.11: Duplicate list1_5 so the user can use the restore list function
	for i=1,table.getn(list1_5) do
		table.insert(olist1_5,list1_5[i])
	end

	osClear()
	cprint(title,show,sel,hide,repair,table.getn(list1_5) .. msg1_5HomebrewsInTotal)

  while true do
	  Pad:Update()
		if Pad.tLd or Pad.Ay>analogDeadZone then
			-- Pushed DOWN (D-pad) or Pushed DOWN (Analog Stick)
			if sel<table.getn(show) then
				sel=sel+1
				if Pad.Rd and move then
					-- Pushed CROSS
					swap(show,sel,sel-1)
				end
			else -- We have reached the last item in the list (going down!)...
				sel=1
				if Pad.Rd and move then
					-- Pushed CROSS
					tmp=table.remove(show) -- Remove the last item from list
					table.insert(show,1,tmp) -- So jump back to the first item on the next move!
				end
			end
			cprint2(title,show,sel,move,hide,repair,msgEditDisabled,msgManualSort)

		elseif Pad.tLu or Pad.Ay<(analogDeadZone*-1) then
			-- Pushed UP (D-pad) or Pushed UP (Analog Stick)
			if sel>1 then
				sel=sel-1
				if Pad.Rd and move then
					-- Pushed CROSS
					swap(show,sel,sel+1)
				end
			else -- We have reached the first item in the list (going up!)...
				sel=table.getn(show)
				if Pad.Rd and move then
					-- Pushed CROSS
					tmp=table.remove(show,1)
					table.insert(show,tmp) -- So jump to the last item on the next move!
				end
			end
			cprint2(title,show,sel,move,hide,repair,msgEditDisabled,msgManualSort)

		elseif Pad.Ax<(analogDeadZone*-1) then
			-- Pushed LEFT (Analog Stick)
			ans=confirm(msgYNQuitOgraniser)
			if ans then
				System.Quit() -- Requires Lua Player .17dk_2 / .20 MOD and above to work
			end
			return nil

		--elseif Pad.Ax>analogDeadZone then
			-- Pushed RIGHT (Analog Stick)

		elseif Pad.tLl then
			-- Pushed LEFT
			title=list1_0title
			show=list1_0
			move=false
			if sel>table.getn(show) then sel=1 end
			cprint(title,show,sel,hide,repair,table.getn(list1_0) .. msg1_0HomebrewsInTotal)

		elseif Pad.tLr then
			-- Pushed RIGHT
			title=list1_5title
			show=list1_5
			move=true
			if sel>table.getn(show) then sel=1 end
			cprint(title,show,sel,hide,repair,table.getn(list1_5) .. msg1_5HomebrewsInTotal)

		elseif Pad.tRu then
			-- Pushed TRIANGLE
			if move then
				-- Hoshizora 2006.07.07: Added SORT function, ASCENDING order
				tableSortAsc(list1_5)
				cprint(title,show,sel,hide,repair,msgAutoSort .. 'ascending order')
			else
				cprint(title,show,sel,hide,repair,msgEditDisabled)
			end

		elseif Pad.tRl then
			-- Pushed SQUARE
			if move then
				-- Hoshizora 2006.07.07: Added SORT function, DESCENDING order
				tableSortDesc(list1_5)
				cprint(title,show,sel,hide,repair,msgAutoSort .. 'descending order')
			else
				cprint(title,show,sel,hide,repair,msgEditDisabled)
			end
		
		elseif Pad.tRr then
			-- Pushed CIRCLE
			if move then
				-- Hoshizora 2006.07.07: Change button function to Restore Original Listing
				restoreOriginalList(list1_5,olist1_5)
				cprint(title,show,sel,hide,repair,msgRestoreOriginalList)
			else
				cprint(title,show,sel,hide,repair,msgEditDisabled)
			end

		elseif Pad.tRd then
			-- Pushed CROSS
			cprint2(title,show,sel,move,hide,repair,msgEditDisabled,msgManualSort)

		elseif Pad.tL1 then
			-- Pushed L
			if move then -- Toggle mode
				if hide then
					hide=false
					msg=msgDispShowOption
				else
					hide=true
					msg=msgDispHideOption
				end
			end
			cprint2(title,show,sel,move,hide,repair,msgEditDisabled,msg)

		elseif Pad.tR1 then
			-- Pushed R
			if move then -- Toggle mode
				if repair==0 then
					repair=1
					msg=msgIconRepairOption
				elseif repair==1 then
					repair=2
					msg=msgIconRestoreOption
				else
					repair=0
					msg=msgIconNoChangeOption
				end
			end
			cprint2(title,show,sel,move,hide,repair,msgEditDisabled,msg)

		elseif Pad.tSl then
			-- Pushed Select
			-- Hoshizora 2006.07.08: Modified screen capture file save location
			screen:save(screencapdir)
			cprint(title,show,sel,hide,repair,msgScreenCaptured)

		elseif Pad.tSt then
			-- Pushed Start
			-- Hoshizora 2006.07.08: Check battery life before commiting changes
			if System.powerIsLowBattery() and not System.powerIsPowerOnline() then
				cprint(title,show,sel,hide,repair,msgIsLowBattery)
			elseif title==list1_0title then
				cprint(title,show,sel,hide,repair,msgEditDisabled)
			else
				ans=confirm(msgYNCommitChanges)
				if ans then return list1_5,repair end
				return nil
			end
		end
		osClear()
	end
end